package de.opalpiraten.scenepainter.gui;
import java.awt.*;
import java.io.*;    
import javax.swing.*;
import java.util.*;
import java.awt.event.*;  
import javax.swing.tree.*;
import javax.swing.event.*;
import java.beans.*;
import java.net.*;
import de.opalpiraten.scenepainter.sys.*;
import de.opalpiraten.scenepainter.gui.render.*;
import com.threerings.media.image.*;

/**
 * This Editor is the GUI frontend for the PictureSceneElement class
 */
public class PictureSceneElementEditor extends JPanel implements 
ListSelectionListener, TreeSelectionListener {

  /**
   * List of available images
   */
  private JList pix = new JList();
  
  /**
   * For recoloring
   */
  private JTree coltree= new JTree();
  
  /**
   * Root node name for the colors
   */
  private static final String COLORROOT = "Colors";
  
  
  /**
   * Reference to the scene showing
   */
  private Scene scene;
  
  /**
   * Reference to the project
   */
  private Project project;
  
  /**
   * Reference to tthe container, this editor resides in
   */
  private SceneElementEditor parent;
  
  public PictureSceneElementEditor(SceneElementEditor see) {
    
    setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
    parent = see;
    
    JScrollPane jsp0 = new JScrollPane(pix);
    JScrollPane jsp1 = new JScrollPane(coltree);
    jsp0.setPreferredSize(jsp0.getPreferredSize());
    jsp1.setPreferredSize(jsp1.getPreferredSize());
    
    
    JTabbedPane tab = new JTabbedPane(JTabbedPane.BOTTOM);
    tab.add(Dict.get("gui.picturesceneelementeditor.tab.pix"),jsp0);
    tab.add(Dict.get("gui.picturesceneelementeditor.tab.colors"),jsp1);
    
    add(tab);
    
    pix.addListSelectionListener(this);
    coltree.addTreeSelectionListener(this);
    coltree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
  }
  
  /**
   * Replace scene currently being edited
   */
  public void replaceScene(Scene s) {
    scene=s;
  }
  
  public void replaceProject(Project p) {
    project=p;
    pix.setListData(p.listPictures());

    ColorPository cpos = p.getColorPository();
    DefaultMutableTreeNode top = new DefaultMutableTreeNode(COLORROOT);
    Iterator clazzes = cpos.enumerateClasses();
    Vector v = new Vector();
    while (clazzes.hasNext()) {
      ColorPository.ClassRecord clazz = (ColorPository.ClassRecord)clazzes.next();
      ColorPository.ColorRecord[] crec= cpos.enumerateColors(clazz.name);
      for (int i=0;i<crec.length;i++) {
        v.add(clazz.name+"/"+crec[i].name);
      }
    }
    String tmp[] = new String[v.size()];
    v.copyInto(tmp);
    Arrays.sort(tmp);
    for (int i=0;i<tmp.length;i++)  {
      Util.sortIntoTree(top,tmp[i].split("/"));
    }
    coltree.setModel(new DefaultTreeModel(top));
  }
  
  public void valueChanged(ListSelectionEvent e) {
    if (pix.getSelectedIndex()<0 || e.getValueIsAdjusting()) return;
    try {
      PictureSceneElement pse = new PictureSceneElement();
      pse.setResource((String)pix.getSelectedValue());
      pse.setProject(project);
      
      
      if (parent.isCreating() || scene.getSelectedIndex()<0) {
        scene.add(pse);
        scene.setSelectedIndex(0);
      }
      else {
        SceneElement se = scene.getSelected();
        if (se!=null) {
          pse.setPosition(se.getPosition());
          pse.setAlpha(se.getAlpha());
        }
        scene.replace(scene.getSelectedIndex(),pse);
      }
    }
    catch (Exception exp) {
      // truly lazy way to do it.
    }
    pix.clearSelection();
  }
  
  public void valueChanged(TreeSelectionEvent e) {
    try {
      PictureSceneElement pse = (PictureSceneElement)scene.getSelected();
      TreePath[] tp = coltree.getSelectionPaths();
      Hashtable colz = new Hashtable();
      for (int i=0;i<tp.length;i++) {
        Object[] tmp = tp[i].getPath();
        if (tmp.length!=3) break;
        colz.put(tmp[1].toString(),tmp[2].toString());
      }
      
      pse.setZations(colz);
      scene.replace(scene.getSelectedIndex(),pse);
    }
    catch (Exception exp) {
      // Lazy sanity checking
      //exp.printStackTrace();
    }
  }
  
}